/*
  Name: map.h
  Author: Master Lenman
  Project: SpeedHack 2009
  Date: 10.07.2009
  Description: The mapclass for the game
*/

#ifndef HEADER_MAP
#define HEADER_MAP

#include <list>
#include <string>
#include <allegro.h>

#include "game.h"

class cobject;

typedef std::list<cobject*> objList;
typedef std::list<cobject*>::iterator objListIter;

class weapon
{
public:
	weapon();
	~weapon();
	
	void load(int id);
	void unload();
	
	BITMAP *gfx;
	int id;
	int dmg;
	int fireDmg;
	int fireRate;
	int coolDown;
	std::string name;
	
	int ammo;
};

const int AMMOVALUE = 1200;
const int MAXINVENTORY = 8;
extern int globalAmmo;
extern weapon weapons[5];
extern BITMAP *weaponFire[2];

const int INVENTORYSIZE = 10;

class cmap
{
private:
	int sizeX, sizeY;
	int timeLimit, timeLeft;
	int entranceX, entranceY;
	int clockColorCounter;
	
	BITMAP *tilesetGfx;
	BITMAP *entryGfx;
	BITMAP *icons[2];
	int floorX, floorY, floorSizeX, floorSizeY;
	bool leftDropzone;
	bool firefighterUnlocked;
	bool lastLevel;
	int inventoryItems;
	
	objList objectTemplates;
	objList objects;
	
	cobject *player;
	cobject *fireTemplate;
	
	std::string introMovie;
	
	gameInfos *infos, *totalStats;
	
	int targets[4];
	//persons, food, medikits, weapons, fires, enemies
	int availible[6];
	
	cobject *findTemplate(std::string name);
		
public:
	cmap();
	~cmap();
	
	void load(std::string path, gameInfos *tInfos, int weaponsUnlocked, bool firefighterUnl, bool lastLevel, gameInfos totals);
	void draw();
	void drawOverview(BITMAP *targetBmp, int targetPosX, int targetPosY, int targetSizeX, int targetSizeY);
	int tick();
	void move(int dx, int dy);
	void shootBullet(int startX, int startY, int dir, cobject *shooter);
	bool useAmmo();
	bool isFirefighter();
	void pickupObject(cobject *object);
	
	cobject *getPlayer();
	
	objList &getObjects();
	
	void spawnCreature(int posX, int posY, int type);
	
	cobject *checkCollision(cobject *who, int newPosX, int newPosY);
};

#endif
