/*
  Name: global.cpp
  Author: Master Lenman
  Project: SpeedHack 2009
  Date: 09.07.2009
  Description: This file contains the global class,
	  that contains all globally accessible variables
*/

#include <allegro.h>
#include <fstream>
#include <string>
#include <alpng.h>

#include "global.h"
#include "errors.h"
#include "main.h"
#include "iniReader.h"
#include "keyStatesClass.h"
#include "menues.h"
#include "cmap.h"

using namespace std;

const float Pi = 3.1415926535897932384626433832795f;

globals global;

void globals::load()
{
	iniReader readfile;
	if(!readfile.load("settings.ini"))
		firstTime = true;
	
	readfile.setCategory("settings");
	
	#ifdef __DEBUG__
	bool autoFullScreen = false;
	#else
	bool autoFullScreen = true;
	#endif
	
	if(readfile.getBValue("fullscreen", autoFullScreen))
	{
		curmode = GFX_AUTODETECT_FULLSCREEN;
	}
	else
	{
		curmode = GFX_AUTODETECT_WINDOWED;
	}
	vsync = readfile.getBValue("vsync", true);
	shotnumber = readfile.getIValue("screenshotnumber", 0);
	
	readfile.setCategory("controls");
	
	keyLeft = getKeyByName(readfile.getValue("left"), KEY_LEFT);
	keyRight = getKeyByName(readfile.getValue("right"), KEY_RIGHT);
	keyUp = getKeyByName(readfile.getValue("up"), KEY_UP);
	keyDown = getKeyByName(readfile.getValue("down"), KEY_DOWN);
	keyStrafe = getKeyByName(readfile.getValue("strafe"), KEY_ALT);
	keyShoot = getKeyByName(readfile.getValue("shoot"), KEY_LCONTROL);
	keyChangeWeapon = getKeyByName(readfile.getValue("change_weapon"), KEY_RCONTROL);
	keyMedipack = getKeyByName(readfile.getValue("medipack"), KEY_ENTER);
}

void globals::save()
{
	iniWriter writefile;
	
	writefile.addCategory("settings");
	writefile.addEntry("fullscreen", curmode == GFX_AUTODETECT_FULLSCREEN || curmode == GFX_AUTODETECT);
	writefile.addEntry("vsync", vsync);
	writefile.addEntry("screenshotnumber", shotnumber);
	
	writefile.addCategory("controls");
	writefile.addEntry("left", keyNames[keyLeft]);
	writefile.addEntry("right", keyNames[keyRight]);
	writefile.addEntry("up", keyNames[keyUp]);
	writefile.addEntry("down", keyNames[keyDown]);
	writefile.addEntry("strafe", keyNames[keyStrafe]);
	writefile.addEntry("shoot", keyNames[keyShoot]);
	writefile.addEntry("change_weapon", keyNames[keyChangeWeapon]);
	writefile.addEntry("medipack", keyNames[keyMedipack]);
	
	writefile.save("settings.ini");
}

void globals::unload()
{
	destroy_bitmap(doublebuffer);
	destroy_font(size8);
	destroy_font(size12);
	destroy_font(size12bold);
	destroy_font(size16);
	destroy_font(size20);
	destroy_bitmap(mainmenu);
	menu::menuBgDestroy();
	destroy_bitmap(weaponFire[0]);
	destroy_bitmap(weaponFire[1]);
    for(int i = 0; i < 5; ++i)
    {
		weapons[i].unload();
	}
	save();
}

void globals::init()
{
	//Initialise variables that do not need to be
	//loaded from an external file

	firstTime = false;

	//Load variables from external files
	load();

	//Set color depth
	set_color_depth(32);

	//Set a graphics mode
	setMode(curmode);
	
	alpng_init();

	colors::white = makecol(0xFF, 0xFF, 0xFF);
	colors::black = 0;
	colors::red = makecol(0xFF, 0x00, 0x00);
	colors::green = makecol(0x00, 0xFF, 0x00);
	colors::blue = makecol(0x00, 0x00, 0xFF);
	colors::pink = makecol(0xFF, 0x00, 0xFF);
	colors::cyan = makecol(0x00, 0xFF, 0xFF);
	colors::yellow = makecol(0xFF, 0xFF, 0x00);

	doublebuffer = createBitmap(SCREEN_W, SCREEN_H);
	mainmenu = createBitmap(SCREEN_W, SCREEN_H);
	clear_to_color(mainmenu, 0);

	size8 = loadBitmapFont("media/fonts/tahoma8.pcx");
	size12 = loadBitmapFont("media/fonts/tahoma12.pcx");
	size12bold = loadBitmapFont("media/fonts/tahoma12bold.pcx");
	size16 = loadBitmapFont("media/fonts/tahoma16.pcx");
	size20 = loadBitmapFont("media/fonts/tahoma20.pcx");

	font = size8;
	
    gui_mouse_focus = 0;
    
    for(int i = 0; i < 5; ++i)
    {
		weapons[i].load(i);
	}
	weaponFire[0] = loadBitmap("media/weapons/fire.png");
	weaponFire[1] = loadBitmap("media/weapons/extinguisher.png");

}

//Toggle between Fullscreen and windowed mode
void globals::toggleMode()
{
	//If mode is auto, make curmode WINDOWED so it will be changed to fullscreen
	if(!curmode)
#ifndef __DEBUG__
		curmode = GFX_AUTODETECT_WINDOWED;
#else
		curmode = GFX_AUTODETECT_FULLSCREEN;
#endif
	curmode ^= 3; //XOR with 3 to switch between graphic modes

	setMode(curmode);
}

namespace colors
{
	int white, red, green, blue, black, yellow, pink, cyan;
};
