/*
  Name: menues.cpp
  Author: Master Lenman
  Project: SpeedHack 2009
  Date: 09.07.2009
  Description: All menu functions are stored in this file
*/

#include <allegro.h>
#include <string>
#include <vector>
#include <list>
#include <fstream>

#include "main.h"
#include "global.h"
#include "menues.h"
#include "errors.h"
#include "keyStatesClass.h"
#include "cobject.h"

using namespace std;

class moviechar
{
private:
	int dir;
	BITMAP *gfx;
	bool character;
	int posX, posY;
	int newPosX, newPosY;
	int speed;
	int animNo;
	int animTicks;
	bool show;
	bool destroyGfx;
	int id;
	
public:
	moviechar()
	{
		dir = 2;
		gfx = 0;
		posX = posY = 0;
		newPosX = newPosY = 0;
		animNo = 1;
		show = true;
		character = true;
		destroyGfx = true;
	}
	
	~moviechar()
	{
		if(destroyGfx && gfx)
		{
			destroy_bitmap(gfx);
		}
	}
	
	int getPosX() const
	{
		return posX;
	}
	
	int getPosY() const
	{
		return posY;
	}
	
	void setCharacter(bool newchar)
	{
		character = newchar;
	}
	
	void draw(int bgX, int bgY, BITMAP *target = global.doublebuffer)
	{
		if(show && gfx)
		{
			if(character)
			{
				masked_blit(gfx, target, 24 * animNo, 32 * dir, posX - 12 + bgX, posY - 16 + bgY, 24, 32);
			}
			else
			{
				masked_blit(gfx, target, 0, 0, posX - gfx->w / 2 + bgX, posY - gfx->h / 2 + bgY, gfx->w, gfx->h);
			}
		}
	}
	
	void setGfx(BITMAP *ngfx)
	{
		gfx = ngfx;
		destroyGfx = false;
	}
	
	void loadGfx(string path)
	{
		gfx = loadBitmap(path);
	}
	
	void setDir(int newdir)
	{
		dir = newdir;
	}
	
	void setPos(int newposx, int newposy)
	{
		posX = newposx;
		posY = newposy;
	}
	
	void setShow(bool newshow)
	{
		show = newshow;
	}
	
	void setTarget(int targetX, int targetY, int nspeed)
	{
		newPosX = targetX;
		newPosY = targetY;
		animTicks = 0;
		speed = nspeed;
		if(speed < 0)
		{
			speed = 1;
		}
		//allegro_message("Driving to %i/%i with speed %i", targetX, targetY, speed);
	}
	
	bool moveChar()
	{
		if(animNo == 1)
		{
			animNo = 0;
		}
		++animTicks;
		if((animTicks % 12) == 0)
		{
			if(animNo == 0)
			{
				animNo = 2;
			}
			else if(animNo == 2)
			{
				animNo = 0;
			}
		}
		if(speed == 1 || speed == 0 || (animTicks % speed) == 0)
		{
			int limit = 1;
			if(speed == 0)
			{
				limit = 2;
			}
			for(int i = 0; i < limit; ++i)
			{
				if(newPosX > posX)
				{
					++posX;
				}
				else if(newPosX < posX)
				{
					--posX;
				}
				if(newPosY > posY)
				{
					++posY;
				}
				else if(newPosY < posY)
				{
					--posY;
				}
				if(newPosX == posX && newPosY == posY)
				{
					animTicks = 0;
					animNo = 1;
					return true;
				}
			}
		}
		return false;
	}
	
	void setId(int newid)
	{
		id = newid;
	}
	
	int getId()
	{
		return id;
	}
};

BITMAP *menuBgTileset = 0;
BITMAP *menuBgChars[4] = { 0, 0, 0, 0 };
BITMAP *menuBgTitle = 0;
list<moviechar*> menuBgObjects;

vector<string> splitParams(string params)
{
	vector<string> retVal;
	string::size_type nextPos;
	while((nextPos = params.find(' ')) != string::npos)
	{
		retVal.push_back(params.substr(0, nextPos));
		params = params.substr(nextPos + 1);
	}
	retVal.push_back(params);
	
	return retVal;
}

moviechar *findChar(const vector<moviechar*> &source, int searchid)
{
	if(searchid < 1)
	{
		return 0;
	}
	for(int i = 0; i < source.size(); ++i)
	{
		if(source[i]->getId() == searchid)
		{
			return source[i];
		}
	}
	return 0;
}

int menu::main(int selection)
{
	const int BUTTONS = 4;
	string labels[BUTTONS] = { "Start new game", "Continue game", "View highscore", "Exit" };
	bool states[BUTTONS] = { true, true, false, true };
	states[1] = fileExists("media/savegame.ini");
	return stdMenu(selection, BUTTONS, labels, states);
}

int menu::showIngameMenu()
{
	blit(global.doublebuffer, global.mainmenu, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
	masked_blit(menuBgTitle, global.mainmenu, 0, 0, (SCREEN_W - menuBgTitle->w) / 2, 10, menuBgTitle->w, menuBgTitle->h);
	const int BUTTONS = 2;
	string labels[BUTTONS] = { "Continue", "Exit game" };
	bool states[BUTTONS] = { true, true };
	return stdMenu(0, BUTTONS, labels, states, global.size20, 75, 0, 0, global.mainmenu, 0);
}

void menu::showmovie(string moviepath)
{
	ifstream readfile(moviepath.c_str());
	
	BITMAP *background = 0;
	
	string nextline;
	vector<string> commands;
	vector<string> parameters;
	while(getline(readfile, nextline, '\n'))
	{
		if(nextline == "")
		{
			continue;
		}
		if(nextline[0] == '!')
		{
			continue;
		}
		string::size_type pos = nextline.find(' ');
		
		string command;
		if(pos == string::npos)
		{
			command = nextline;
			nextline = "";
		}
		else
		{
			command = nextline.substr(0, pos);
			nextline = nextline.substr(pos + 1);
		}
		//	allegro_message((command + "." + nextline).c_str());
		commands.push_back(command);
		parameters.push_back(nextline);
	}
	
	timerCounter = 0;
	int commandCount = 0;
	bool waiting = false;
	int waitingtime = 0;
	
	vector<string> msgBuffer;
	vector<string> displayMsgBuffer;
	vector<moviechar*> characters;
	
	int bgX = 0, bgY = 0;
	
	bool showMessageBuffer = false;
	bool showMessageTitle = false;
	int movingchar = 0;
	bool run = true;
	while(run)
	{
		while(timerCounter > 0)
		{
			if(!waiting)
			{
				while(true)
				{
					if(commandCount == commands.size())
					{
						run = false;
						break;
					}
					string nextcommand = commands[commandCount];
					string nextparameter = parameters[commandCount];
					++commandCount;
					
					if(nextcommand == "line")
					{
						msgBuffer.push_back(nextparameter);
					}
					else if(nextcommand == "showlines")
					{
						displayMsgBuffer = msgBuffer;
						msgBuffer.clear();
						showMessageBuffer = true;
						waiting = true;
						break;
					}
					else if(nextcommand == "showtitle")
					{
						displayMsgBuffer = msgBuffer;
						msgBuffer.clear();
						showMessageTitle = true;
						waiting = true;
						break;
					}
					else if(nextcommand == "loadbg")
					{
						if(background)
						{
							destroy_bitmap(background);
						}
						vector<string> params = splitParams(nextparameter);
						if(params.size() > 3)
						{
							continue;
						}
						background = loadBitmap(params[0]);
						if(params.size() == 1)
						{
							bgX = (SCREEN_W - background->w) / 2;
							bgY = (SCREEN_H - background->h) / 2;
						}
						if(params.size() == 2)
						{
							bgX = (SCREEN_W - background->w) / 2;
							bgY = str2i(params[1], 0);
						}
						if(params.size() == 3)
						{
							bgX = str2i(params[1], 0);
							bgY = str2i(params[2], 0);
						}
					}
					else if(nextcommand == "createsupermarket")
					{
						//Load template file and copy random junks onto the background gfx
						
						//Ein Feld: 47x57
						//Startpos frs erste feld 160x54; bild 350x170
						BITMAP *supermarket = loadBitmap("media/movies/supermarche.png");
						
						const int FIELDX = 47, FIELDY = 57;
						
						int bgWall = rand() % 3;
						int lastFg = -1;
						for(int i = 0; i < 5; ++i)
						{
							masked_blit(supermarket, background, bgWall * FIELDX, 0, 160 + FIELDX * i, 54, FIELDX, FIELDY);
							int fgWall;
							do
							{
								fgWall = rand() % 10 + 3;
							} while(fgWall == lastFg);
							if(i == 1)
							{
								fgWall = 13;
							}
							else
							{
								lastFg = fgWall;
							}
							masked_blit(supermarket, background, fgWall * FIELDX, 0, 160 + FIELDX * i, 54, FIELDX, FIELDY);
						}
						
						destroy_bitmap(supermarket);
					}
					else if(nextcommand == "pause")
					{
						waiting = true;
						waitingtime = str2i(nextparameter, 0);
						if(waitingtime <= 0)
						{
							waiting = false;
							continue;
						}
						break;
					}
					else if(nextcommand == "wait")
					{
						waiting = true;
						break;
					}
					else if(nextcommand == "char")
					{
						vector<string> params = splitParams(nextparameter);
						if(params.size() != 5)
						{
							continue;
						}
						int iparams[4];
						for(int i = 1; i < 5; ++i)
						{
							iparams[i - 1] = str2i(params[i], 0);
						}
						
						moviechar *newchar = new moviechar();
						
						newchar->loadGfx(params[0]);
						newchar->setId(iparams[0]);
						newchar->setPos(iparams[1], iparams[2]);
						newchar->setDir(iparams[3]);
						
						characters.push_back(newchar);
					}
					else if(nextcommand == "object")
					{
						vector<string> params = splitParams(nextparameter);
						if(params.size() != 4)
						{
							continue;
						}
						int iparams[3];
						for(int i = 1; i < 4; ++i)
						{
							iparams[i - 1] = str2i(params[i], 0);
						}
						
						moviechar *newchar = new moviechar();
						
						newchar->loadGfx(params[0]);
						newchar->setId(iparams[0]);
						newchar->setPos(iparams[1], iparams[2]);
						newchar->setCharacter(false);
						
						characters.push_back(newchar);
					}
					else if(nextcommand == "hide")
					{
						moviechar *curchar = findChar(characters, str2i(nextparameter, 0));
						if(curchar)
						{
							curchar->setShow(false);
						}
					}
					else if(nextcommand == "dir")
					{
						vector<string> params = splitParams(nextparameter);
						if(params.size() != 2)
						{
							continue;
						}
						moviechar *curchar = findChar(characters, str2i(params[0], 0));
						if(curchar)
						{
							curchar->setDir(str2i(params[1], 0));
						}
					}
					else if(nextcommand == "show")
					{
						vector<string> params = splitParams(nextparameter);
						if(params.size() != 4)
						{
							continue;
						}
						moviechar *curchar = findChar(characters, str2i(params[0], 0));
						if(curchar)
						{
							curchar->setShow(true);
							curchar->setPos(str2i(params[1], 0), str2i(params[2], 0));
							curchar->setDir(str2i(params[3], 0));
						}
					}
					else if(nextcommand == "move")
					{
						vector<string> params = splitParams(nextparameter);
						if(params.size() != 4)
						{
							continue;
						}
						moviechar *curchar = findChar(characters, str2i(params[0], 0));
						if(curchar)
						{
							curchar->setShow(true);
							curchar->setTarget(str2i(params[1], 0), str2i(params[2], 0), str2i(params[3], 1));
							movingchar = curchar->getId();
							waiting = true;
							break;
						}
					}
				}
			}
			else
			{
				if(movingchar)
				{
					moviechar *curchar = findChar(characters, movingchar);
					if(curchar->moveChar())
					{
						waiting = false;
						movingchar = 0;
					}
				}
				else if(waitingtime > 0)
				{
					--waitingtime;
					if(!waitingtime)
					{
						waiting = false;
					}
				}
				else if(keyStates[KEY_ENTER] || keyStates[KEY_SPACE])
				{
					waiting = false;
					showMessageTitle = false;
					showMessageBuffer = false;
				}
			}
			if(keyStates[KEY_ESC])
			{
				run = false;
				break;
			}
			--timerCounter;
		}
		//Draw:
		const int LineHeight = 19;
		clear_to_color(global.doublebuffer, 0);
		if(background && !showMessageTitle)
		{
			blit(background, global.doublebuffer, 0, 0, bgX, bgY, background->w, background->h);
		}
		for(int i = 0; i < characters.size(); ++i)
		{
			characters[i]->draw(bgX, bgY);
		}
		if(showMessageTitle && displayMsgBuffer.size() > 0)
		{
			textout_centre_ex(global.doublebuffer, global.size20, displayMsgBuffer[0].c_str(), SCREEN_W / 2, SCREEN_H / 2, colors::white, -1);
			for(int i = 1; i < displayMsgBuffer.size(); ++i)
			{
				textout_centre_ex(global.doublebuffer, global.size12, displayMsgBuffer[1].c_str(), SCREEN_W / 2, SCREEN_H / 2 + LineHeight * (i + 2), colors::white, -1);
			}
		}
		if(showMessageBuffer)
		{
			int startHeightSize = displayMsgBuffer.size();
			if(startHeightSize < 4)
			{
				startHeightSize = 4;
			}
			int startHeight = SCREEN_H - LineHeight * startHeightSize - 8;
			for(int i = 0; i < displayMsgBuffer.size(); ++i)
			{
				textout_ex(global.doublebuffer, global.size12, displayMsgBuffer[i].c_str(), 13, startHeight + LineHeight * i, colors::white, -1);
			}
		}
		if(waiting && !waitingtime && !movingchar)
		{
			triangle(global.doublebuffer, 750, 589, 789, 569, 750, 549, colors::white);
		}
		show();
	}
	
	for(int i = 0; i < characters.size(); ++i)
	{
		delete characters[i];
	}
	
	readfile.close();
}

void menu::drawMenuBg(BITMAP *target)
{
	if(!menuBgTileset)
	{
		menuBgTileset = loadBitmap("media/tilesets/menuBg.png");
	}
	if(!menuBgTitle)
	{
		menuBgTitle = loadBitmap("media/fonts/title.png");
	}
	if(!menuBgChars[0])
	{
		menuBgChars[0] = loadBitmap("media/chars/pig.png");
	}
	if(!menuBgChars[1])
	{
		menuBgChars[1] = loadBitmap("media/chars/dragon.png");
	}
	if(!menuBgChars[2])
	{
		menuBgChars[2] = loadBitmap("media/chars/basher.png");
	}
	if(!menuBgChars[3])
	{
		menuBgChars[3] = loadBitmap("media/chars/golem.png");
	}
	if(menuBgObjects.size() == 0)
	{
		while(menuBgObjects.size() < 30)
		{
			moviechar* newChar = new moviechar();
			newChar->setDir(3);
			newChar->setPos(rand() % 800, rand() % 600);
			newChar->setGfx(menuBgChars[rand() % 4]);
			newChar->setTarget(-24, newChar->getPosY(), 2 + rand() % 3);
			menuBgObjects.push_back(newChar);
		}
	}
	
	clear_to_color(global.mainmenu, 0);
	for(int x = 0; x < SCREEN_W / 16 + 1; ++x)
	{
		for(int y = 0; y < SCREEN_H / 16 + 1; ++y)
		{
			blit(menuBgTileset, target, 0, 0, x * 16, y * 16, 16, 16);
		}
	}
	
	for(list<moviechar*>::iterator iter = menuBgObjects.begin(); iter != menuBgObjects.end(); ++iter)
	{
		moviechar *curChar = *iter;
		if(curChar->moveChar())
		{
			curChar->setPos(824, rand() % 600);
			curChar->setTarget(-24, curChar->getPosY(), 2 + rand() % 3);
		}
		curChar->draw(0, 0, target);
	}
	
	masked_blit(menuBgTitle, target, 0, 0, (SCREEN_W - menuBgTitle->w) / 2, 10, menuBgTitle->w, menuBgTitle->h);
	
	textout_right_ex(target, global.size12, allegro_id, SCREEN_W - 3, SCREEN_H - 6 - 2 * text_height(global.size12), colors::white, -1);
	textout_right_ex(target, global.size12, (VERSIONSTRING
#ifdef __DEBUG__
		+ ", debug version"
#endif
		).c_str(), SCREEN_W - 3, SCREEN_H - 3 - text_height(global.size12), colors::white, -1);
}

void menu::menuBgDestroy()
{
	if(menuBgTileset)
	{
		destroy_bitmap(menuBgTileset);
	}
	if(menuBgTitle)
	{
		destroy_bitmap(menuBgTitle);
	}
	for(int i = 0; i < 4; ++i)
	{
		if(menuBgChars[i])
		{
			destroy_bitmap(menuBgChars[i]);
		}
	}
	for(list<moviechar*>::iterator iter = menuBgObjects.begin(); iter != menuBgObjects.end(); ++iter)
	{
		delete *iter;
	}
}

void menu::showBriefing(ifstream &readFile, int *targets, int *availible, int timeLimit)
{
	vector<string> briefingText;
	string nextLine;
	getline(readFile, nextLine, '\n');
	while(getline(readFile, nextLine, '\n') && nextLine != "endbriefing")
	{
		briefingText.push_back(nextLine);
	}
	
	bool run = true;
	while(run)
	{
		while(timerCounter > 0)
		{
			if(keyStates[KEY_SPACE] || keyStates[KEY_ENTER])
			{
				run = false;
				break;
			}
			
			clear_to_color(global.doublebuffer, colors::white);
			
			textout_centre_ex(global.doublebuffer, global.size20, briefingText[0].c_str(), 250, 12, 0, -1);
			for(int i = 1; i < briefingText.size(); ++i)
			{
				textout_ex(global.doublebuffer, global.size12, briefingText[i].c_str(), 15, 30 + i * (text_height(global.size12) + 3), 0, -1);
			}
			
			rectfill(global.doublebuffer, 500, 0, 510, 599, colors::black);
			//rectfill(global.doublebuffer, 500, 295, 799, 305, colors::black);
			
			//textout_centre_ex(global.doublebuffer, global.size12bold, "Map", 650, 15, 0, -1);
			textout_centre_ex(global.doublebuffer, global.size12bold, "Infos", 650, 15, 0, -1);
			int textPos = 15 + (text_height(global.size12bold) + 3);
			int lineHeight = text_height(global.size12) + 3;
			const string ITEMNAMES[6] = { "People", "Food", "Medikits", "Weapons", "Enemies", "Fires" };
			if(timeLimit > 0)
			{
				int minutes = timeLimit / 60;
				int seconds = timeLimit - minutes * 60;
				string timeString = "Time limit: " + i2str(minutes) + ":" + i2str(seconds, 2) + " min";
				textout_ex(global.doublebuffer, global.size12, timeString.c_str(), 520, textPos, 0, -1);
				textPos += lineHeight;
			}
			bool shownMinMsg = false;
			for(int i = 0; i < 4; ++i)
			{
				if(targets && targets[i] > 0)
				{
					if(!shownMinMsg)
					{
						textout_ex(global.doublebuffer, global.size12bold, "Requirements:", 520, textPos, 0, -1);
						textPos += lineHeight;
						shownMinMsg = true;
					}
					
					textout_ex(global.doublebuffer, global.size12, (ITEMNAMES[i] + ": " + i2str(targets[i])).c_str(), 520, textPos, 0, -1);
					textPos += lineHeight;
				}
			}
			if(!shownMinMsg)
			{
				textPos += lineHeight;
				textout_ex(global.doublebuffer, global.size20, "Survive!", 600, textPos, 0, -1);
				textPos += lineHeight;
			}
			shownMinMsg = false;
			for(int i = 0; i < 6; ++i)
			{
				if(availible && availible[i] > 0)
				{
					if(!shownMinMsg)
					{
						textout_ex(global.doublebuffer, global.size12bold, "Availible:", 520, textPos, 0, -1);
						textPos += lineHeight;
						shownMinMsg = true;
					}
					
					textout_ex(global.doublebuffer, global.size12, (ITEMNAMES[i] + ": " + i2str(availible[i])).c_str(), 520, textPos, 0, -1);
					textPos += lineHeight;
				}
			}
			
			textout_centre_ex(global.doublebuffer, global.size16, "Your game has been saved!", 250, 569, 0, -1);
			triangle(global.doublebuffer, 750, 589, 789, 569, 750, 549, colors::black);
			show();
			
			--timerCounter;
		}
	}
}

void menu::showEndResult(gameInfos &totals)
{
	bool run = true;
	while(run)
	{
		while(timerCounter > 0)
		{
			if(keyStates[KEY_SPACE] || keyStates[KEY_ENTER])
			{
				run = false;
				break;
			}
			
			clear_to_color(global.doublebuffer, colors::white);
			drawMenuBg(global.doublebuffer);
			
			rect(global.doublebuffer, 200, 150, 600, 450, 0);
			rectfill(global.doublebuffer, 201, 151, 599, 449, colors::white);
			
			textout_centre_ex(global.doublebuffer, global.size20, "Congratulations", 400, 160, colors::red, -1);
			textout_centre_ex(global.doublebuffer, global.size16, "You survived the attack!", 400, 210, 0, -1);
			textout_centre_ex(global.doublebuffer, global.size16, "%i survived with you.", 400, 240, 0, -1);
			
			triangle(global.doublebuffer, 750, 589, 789, 569, 750, 549, colors::red);
			show();
			
			--timerCounter;
		}
	}
}

void menu::showAchievement(int id, string message3)
{
	string message1, message2;
	if(id < 3)
	{
		message1 = "Congratulations! You unlocked a new weapon!";
		message2 = weapons[id + 2].name;
	}
	else if(id == 3)
	{
		message1 = "Congratulations! You extinguished a lot of fires and";
		message2 = "are now a firefighter! You take less damage from fire.";
	}
	else if(id == 4)
	{
		message1 = "For the last level, you get the help of the";
		message2 = "people you saved in the previous levels.";
	}
	clear_to_color(global.doublebuffer, colors::white);
	textout_centre_ex(global.doublebuffer, global.size20, message1.c_str(), 400, 250, 0, -1);
	textout_centre_ex(global.doublebuffer, global.size20, message2.c_str(), 400, 300, 0, -1);
	if(message3.length() > 0)
	{
		textout_centre_ex(global.doublebuffer, global.size20, message3.c_str(), 400, 350, 0, -1);
	}
	triangle(global.doublebuffer, 750, 589, 789, 569, 750, 549, colors::black);
	while(true)
	{
		while(timerCounter > 0)
		{
			if(keyStates[KEY_ENTER] || keyStates[KEY_SPACE])
			{
				return;
			}
			show();
			--timerCounter;
		}
	}
}

void menu::showLevelResults(gameInfos &info, bool total, bool gameOverMsg)
{
	string headline = !total ? "Level statistics" : "Total statistics";
	int lineHeight = text_height(global.size12bold) + 3;
	while(true)
	{
		while(timerCounter > 0)
		{
			clear_to_color(global.doublebuffer, colors::white);
			textout_centre_ex(global.doublebuffer, global.size20, headline.c_str(), 400, 12, 0, -1);
			
			int linePos = 50;
			
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "People saved %i of %i", info.saved[0], info.availibleSaves[0]);
			linePos += lineHeight;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Food collected %i of %i", info.saved[1], info.availibleSaves[1]);
			linePos += lineHeight;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Medipacks collected %i of %i", info.saved[2], info.availibleSaves[2]);
			linePos += lineHeight;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Weapons collected %i of %i", info.saved[3], info.availibleSaves[3]);
			linePos += lineHeight;
			linePos += lineHeight;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Enemies killed %i of %i", info.killed[0], info.availibleKills[0]);
			linePos += lineHeight;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Fires extinguished %i of %i", info.killed[1], info.availibleKills[1]);
			linePos += lineHeight;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Bullets fired: %i / hits: %i",  info.bulletsUsed, info.bulletsHit);
			linePos += lineHeight;
			linePos += lineHeight;
			
			int timeUsed = info.timeUsed;
			if(!total)
			{
				timeUsed /= 60;
			}
			int minutes = timeUsed / 60;
			int seconds = timeUsed - minutes * 60;
			textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Time taken %imin %is",  minutes, seconds);
			linePos += lineHeight;
			linePos += lineHeight;
			if(info.endFight > 0)
			{
				textprintf_centre_ex(global.doublebuffer, global.size12bold, 400, linePos, 0, -1, "Survivors in last battle %i of %i", info.killed[1], info.availibleKills[1]);
				linePos += lineHeight;
			}
			
			if(gameOverMsg)
			{
				textout_centre_ex(global.doublebuffer, global.size20, "GAME OVER - YOU LOST!", 400, 500, 0, -1);
			}
			
			
			triangle(global.doublebuffer, 750, 589, 789, 569, 750, 549, colors::black);
			
			show();
			
			if(keyStates[KEY_ENTER] || keyStates[KEY_SPACE])
			{
				return;
			}
			--timerCounter;
		}
	}
}

int menu::stdMenu(const int startValue, const int BUTTONS, const string *labels, bool *states, FONT *toUse, int borders, BITMAP *button, BITMAP *buttonpressed, BITMAP *background, int escapeKey, int col_standard, int col_selected, int col_disabled)
{
	int i;
	int color_selected = makecol(0xFF, 0xFF, 0xFF),
		color_disabled = makecol(0x33, 0x33, 0x33),
		color_standard = makecol(0xAA, 0xAA, 0xAA);
	if(col_selected > -1)
	    color_selected = col_selected;
	if(col_disabled > -1)
	    color_disabled = col_disabled;
	if(col_standard > -1)
	    color_standard = col_standard;
	if(escapeKey < 0)
	{
		escapeKey = BUTTONS - 1;
	}
	bool hasActive = false;
	for(i = 0; i < BUTTONS; ++i)
	{
		if(states[i])
		{
			hasActive = true;
			break;
		}
	}
	if(!hasActive)
		states[BUTTONS - 1] = true;
	bool destroyButton = false, destroyButtonPressed = false;
	bool useRenderBackground = false;
	if(!background)
	{
		background = global.mainmenu;
		useRenderBackground = true;
	}
	if(!button)
	{
		button = loadBitmap("media/button.png");
		destroyButton = true;
	}
	if(!buttonpressed)
	{
		buttonpressed = loadBitmap("media/buttonpressed.png");
		destroyButtonPressed = true;
	}

	int curSelection = startValue;
	while(!states[curSelection])
		++curSelection;

	int *posY = new int[BUTTONS];
	//int space = (height - (borders * 2) - button->h * BUTTONS) / (BUTTONS - 1);
	int space = SCREEN_H - borders * 2;
	int singleSpace = space / BUTTONS;
	for(i = 0; i < BUTTONS; ++i)
	{
		posY[i] = borders + i * singleSpace + ((singleSpace - button->h) >> 1);
		//posY[i] = (borders >> 1) + i * space + i * button->h;
	}
	int fontHeight = text_height(toUse);

	keyStates[KEY_UP];
	keyStates[KEY_DOWN];
	space = true;
	bool needsUpdate = true;
	while(space)
	{
		if(timerCounter > 0)
		{
			if(keyStates[KEY_UP])// || keyStates[global.keyUp])
			{
				position_mouse(0, 0);
				if(!curSelection)
					curSelection = BUTTONS - 1;
				else
					--curSelection;
				while(!states[curSelection])
				{
					if(!curSelection)
						curSelection = BUTTONS - 1;
					else
						--curSelection;
				}
			}
			if(keyStates[KEY_DOWN])// || keyStates[global.keyDown])
			{
				position_mouse(0, 0);
				++curSelection %= BUTTONS;
				while(!states[curSelection])
				{
					++curSelection %= BUTTONS;
				}
			}
			if(keyStates[KEY_ENTER])// || clickEvent())
				space = false;
			if(keyStates[KEY_ESC])
			{
				curSelection = escapeKey;
				space = false;
			}
				
			needsUpdate = true;
			if(useRenderBackground)
			{
				drawMenuBg(background);
			}
			--timerCounter;
		}
		if(needsUpdate)
		{
			blit(background, global.doublebuffer, 0, 0, 0, 0, background->w, background->h);
			for(i = 0; i < BUTTONS; ++i)
			{
				if(i == curSelection)
				{
					masked_blit(button, global.doublebuffer, 0, 0, (SCREEN_W - buttonpressed->w) >> 1, posY[i], buttonpressed->w, buttonpressed->h);
					textout_centre_ex(global.doublebuffer, toUse, labels[i].c_str(), SCREEN_W >> 1, posY[i] + ((button->h - fontHeight) >> 1), color_selected, -1);
				}
				else
				{
					masked_blit(buttonpressed, global.doublebuffer, 0, 0, (SCREEN_W - button->w) >> 1, posY[i], button->w, button->h);
					if(states[i])
						textout_centre_ex(global.doublebuffer, toUse, labels[i].c_str(), SCREEN_W >> 1, posY[i] + ((button->h - fontHeight) >> 1), color_standard, -1);
					else
						textout_centre_ex(global.doublebuffer, toUse, labels[i].c_str(), SCREEN_W >> 1, posY[i] + ((button->h - fontHeight) >> 1), color_disabled, -1);
				}
			}
			show();
			needsUpdate = false;
		}
		rest(1);
	}

	delete [] posY;
	if(destroyButton)
		destroy_bitmap(buttonpressed);
	if(destroyButtonPressed)
		destroy_bitmap(button);
	return curSelection;
}
